@@ -111,8 +111,11 @@ class NameManager:
     def unique_name(self, candidate):
         no_index_if_possible = True
         if not isinstance(candidate, str):
-            candidate = candidate.__class__.__name__.lower()
-            no_index_if_possible = False
+            if hasattr(candidate, "arguments") and "name" in candidate.arguments:
+                candidate = candidate.arguments["name"]
+            else:
+                candidate = candidate.__class__.__name__.lower()
+                no_index_if_possible = False
         
         index_str = re.search(r"\d*$", candidate).group()
         sl = len(index_str)
@@ -580,17 +583,17 @@ class Cake():
                 response[topping_id] = {"revision":self._topping_revisions[topping_id]["revision"], "content":self._topping_revisions[topping_id]["topping"].html(), "function_call": list(self.functions_to_call.values())}
         return response
     
-    
-    def add(self, topping, name=None, storaged=False):
-        if not isinstance(topping, Topping):
-            from .toppings.automatic import topping_from_object
-            topping = topping_from_object(topping)
-            if topping is None:
-                self.plate.logger.error("Incompatible object.")
-                return None
+    def register_topping(self, topping, name=None, skip_rendering=False):
         name = self.name_manager.unique_name(topping if name is None else name)
         self.toppings[name] = topping
-        self.topping_order.append(name)
+        if not skip_rendering:
+            self.topping_order.append(name)
+        return name
+    
+    def add(self, topping, name=None, storaged=False):
+        assert topping is not None
+        topping = input_topping_converter(topping, self.plate.logger)
+        name = self.register_topping(topping, name=name)
         topping.set_cake(self, uid=f"{self.name}.{name}", name=name)
         self.plate.refresh()
         return topping
@@ -806,6 +809,15 @@ class Pancake():
         self.add(FromFunction(func), name=func.__name__)
         return wrapper
 
+    @property
+    def toppings(self):
+        toppings = self._cake.toppings.copy()
+        try:
+            toppings.pop("magic_card")
+        except:
+            pass
+        return toppings
+
     def __call__(self, *args, **kwargs):
         self.show_message(*args, **kwargs)
 
@@ -967,7 +979,7 @@ class Topping():
         self.cake = None
         self.revision = 0
         self.topping_order = []
-        self.child_toppings = {}
+        self.toppings = {}
         self.function_to_call = None
         self._value = None
         self._args = args
@@ -1021,8 +1033,9 @@ class Topping():
         self.cake = cake
         self.uid = uid
         self.name = name
-        for cname, topping in self.child_toppings.items():
+        for cname, topping in self.toppings.items():
             topping.set_cake(self.cake, uid=f"{self.uid}.{cname}", name=cname)
+            self.cake.register_topping(topping, name=cname, skip_rendering=True)
     
         keys = inspect.signature(self.prepare).parameters.keys()
         kwargs = {k: v for k, v in self.arguments.items() if k in keys}
@@ -1057,14 +1070,17 @@ class Topping():
     def html(self):
         return "<p>[topping]</p>"
     
-    def add(self, item, name:str=None):
-        name = self.name_manager.unique_name(item if name is None else name)
-        self.child_toppings[name] = item
+    def add(self, topping, name:str=None): # At this point self.cake can be None.
+        topping = input_topping_converter(topping)
+        name = self.name_manager.unique_name(topping if name is None else name)
+        self.toppings[name] = topping
         self.topping_order.append(name)
-        item.set_cake(self.cake, uid=f"{self.uid}.{name}", name=name)
+        topping.set_cake(self.cake, uid=f"{self.uid}.{name}", name=name)
+        if self.cake is not None:
+            self.cake.register_topping(topping, name=name, skip_rendering=True)
 
-        item.parent = self
-        return item
+        topping.parent = self
+        return topping
 
     def remove(self, topping_name):
         if topping_name in self.topping_order:
@@ -1121,7 +1137,7 @@ class Topping():
     def child_htmls(self):
         htmls = []
         for name in self.topping_order:
-            htmls.append(self.child_toppings[name].render())
+            htmls.append(self.toppings[name].render())
         return htmls
         
     @property
@@ -1192,7 +1208,7 @@ class Topping():
         for k, v in self.arguments.items():
             args += f",{k}={v}"
         script += "("+args+")"
-        return script
+        return script    
     
     def prepare(self, request: dict):
         return
@@ -1209,6 +1225,13 @@ class Topping():
     def value_getter(self): # Method to be overrode for a custom topping
         return self._value
 
+class Arrangement(Topping):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+    def value_getter(self):
+        return pk_wrapped_dict(lambda: self.cake.plate.logger.error("Cannot set values."), self.toppings)
+
 
 class Card(Topping):
     ROOT_DIV_CLASS = "draggable w3-card"
@@ -1385,7 +1408,7 @@ class Variables():
     
 class MagicPancake():
     SHARED = {}
-    RESERVED = ("SHARED", "plate", "toppings", "_current")
+    RESERVED = ("SHARED", "plate", "_current")
     def __init__(self, plate=None):
         if plate is None:
             cake = Pancake()
@@ -1401,24 +1424,6 @@ class MagicPancake():
             return
         return pancake
     
-    @property
-    def toppings(self):
-        named_toppings = {name: self._current._cake.toppings[name] for name in self._current._cake.topping_order}
-        for k, v in vars(self._current).items():
-            if isinstance(v, Topping):
-                named_toppings[k] = v
-
-        topping_list_dict = {}
-        for name, topping in named_toppings.items():
-            if isinstance(topping, FloatingCard):
-                continue
-            cls = type(topping).__name__
-            if cls not in topping_list_dict:
-                topping_list_dict[cls] = []
-            topping_list_dict[cls].append(name)
-        return topping_list_dict
-        
-    
     def __call__(self, *args, **kwargs):
         self.plate.show_message(*args, **kwargs)
 
@@ -1458,8 +1463,19 @@ class Kitchen(Pancake):
     def show_up(self, request):
         cakes = list(self.plate.pancakes.keys())
         cakes.remove("_Kitchen")
-        self.cakes.set(cakes, header="cake")
+        self.cakes.set({"cake": cakes})
     
     def new_cake(self):
         Pancake(self.plate, name=self.cake_name.value)
-        self.plate.reload()
\ No newline at end of file
+        self.plate.reload()
+
+def input_topping_converter(topping, logger=None):
+    if not isinstance(topping, Topping):
+        from .toppings.automatic import topping_from_object
+        topping = topping_from_object(topping)
+        if topping is None:
+            if logger is not None:
+                logger.error("Incompatible object.")
+            return None
+        return topping
+    return topping
\ No newline at end of file
