@@ -1,5 +1,5 @@
 from .basic import Button, Label, Text, Input, DictInput, Slider, Row, Column
-from .image import ImageBox, ImageCard
+from .image import ImageView, ImageCard
 from .table import Table
-from .scientific import ArrayToImageBox
+from .nparray import ArrayToImageView
 from .automatic import FromFunction, topping_from_object
\ No newline at end of file
